@@ -41,6 +41,9 @@ broker_transport_options = {
     'priority_steps': list(range(task_queue_max_priority + 1))
 }
 
+worker_proc_alive_timeout = 8
+"""The timeout when waiting for a new worker process to start up."""
+
 worker_log_format = "[%(asctime)s: %(levelname)s/%(processName)s/%(threadName)s] %(message)s"  # noqa: E501
 """Custom worker log format that includes the thread name."""
 
@@ -53,8 +56,14 @@ expose_to_public = False
 """Set to True if events meeting the public alert threshold really should be
 exposed to the public."""
 
-lvalert_host = 'lvalert-playground.cgca.uwm.edu'
-"""LVAlert host."""
+igwn_alert_server = None
+"""IGWN alert server: None == DEFAULT_SERVER"""
+
+igwn_alert_noauth = False
+"""IGWN alert server no-authetication"""
+
+igwn_alert_group = 'gracedb-playground'
+"""IGWN alert group."""
 
 gracedb_host = 'gracedb-playground.ligo.org'
 """GraceDB host."""
@@ -80,14 +89,14 @@ email_host = 'imap.gmail.com'
 superevent_d_t_start = {'gstlal': 1.0,
                         'spiir': 1.0,
                         'pycbc': 1.0,
-                        'mbtaonline': 1.0}
+                        'mbta': 1.0}
 """Pipeline based lower extent of superevent segments.
 For cwb and lib this is decided from extra attributes."""
 
 superevent_d_t_end = {'gstlal': 1.0,
                       'spiir': 1.0,
                       'pycbc': 1.0,
-                      'mbtaonline': 1.0}
+                      'mbta': 1.0}
 """Pipeline based upper extent of superevent segments
 For cwb and lib this is decided from extra attributes."""
 
@@ -106,7 +115,7 @@ superevent_default_d_t_end = 1.0
 superevent_far_threshold = 1 / 3600
 """Maximum false alarm rate to consider events superevents."""
 
-preliminary_alert_timeout = 30.0
+preliminary_alert_timeout = 0.0
 """Wait this many seconds for the preferred event to stabilize before issuing a
 preliminary alert."""
 
@@ -122,8 +131,8 @@ producing early warning events: gstlal and spiir."""
 
 preliminary_alert_trials_factor = dict(cbc=5.0, burst=4.0)
 """Trials factor corresponding to trigger categories. For CBC and Burst, trials
-factor is the number of pipelines. CBC pipelines are gstlal, pycbc, mbtaonline
-and spiir. Burst searches are cwb.allsky, cwb.bbh and cwb.imbh."""
+factor is the number of pipelines. CBC pipelines are gstlal, pycbc, mbta and
+spiir. Burst searches are cwb.allsky, cwb.bbh and cwb.imbh."""
 
 early_warning_alert_far_threshold = 1 / (3600 * 24)
 """False alarm rate threshold for early warning alerts."""
@@ -132,17 +141,16 @@ snews_gw_far_threshold = 1 / (3600 * 24)
 """Maximum false alarm rate for a superevent to send out a coincidence alert
 between an external SNEWS alert and the superevent."""
 
+sog_paper_far_threshold = {'gw': 1 / (1000 * 365 * 86400),
+                           'joint': 1 / (10000 * 365 * 86400)}
+"""False alarm rate thresholds for producing a manuscript of speed of gravity
+measurement in low-latency."""
+
 superevent_clean_up_timeout = 270.
 """The orchestrator will wait this many seconds from the time of the
 application of the GCN_PRELIM_SENT label to revise the preferred
 event out of the accumulated events."""
 
-subthreshold_annotation_timeout = 300.
-"""The orchestrator will wait this many seconds from the time of the
-creation of a new superevent to the time that subthreshold superevents
-are annotated. It is expected that the timeout is long enough such
-that there are no more G events being added to the superevent."""
-
 pe_timeout = 345.0
 """The orchestrator will wait this many seconds from the time of the
 creation of a new superevent to the time that parameter estimation begins, in
@@ -151,7 +159,7 @@ case the preferred event is updated with high latency."""
 check_vector_prepost = {'gstlal': [2, 2],
                         'spiir': [2, 2],
                         'pycbc': [2, 2],
-                        'MBTAOnline': [2, 2],
+                        'MBTA': [2, 2],
                         'oLIB': [1.5, 1.5],
                         'LIB': [1.5, 1.5],
                         'CWB': [1.5, 1.5],
@@ -167,7 +175,7 @@ vector check will include in its check. Pipeline dependent."""
 uses_gatedhoft = {'gstlal': True,
                   'spiir': True,
                   'pycbc': True,
-                  'MBTAOnline': True,
+                  'MBTA': True,
                   'oLIB': False,
                   'LIB': False,
                   'CWB': True,
@@ -180,9 +188,10 @@ uses_gatedhoft = {'gstlal': True,
 """Whether or not a pipeline uses gated h(t). Determines whether or not
 the DMT-DQ_VECTOR will be analyzed for data quality."""
 
-llhoft_glob = '/dev/shm/kafka/{detector}_O3/*.gwf'
+llhoft_glob = '/dev/shm/kafka/{detector}_O3ReplayMDC/*.gwf'
 """File glob for playground low-latency h(t) frames. Currently points
-to O3 replay data."""
+to O3 MDC Mock Data Challange data.
+See https://git.ligo.org/emfollow/mock-data-challenge"""
 
 llhoft_channels = {
     'H1:DMT-DQ_VECTOR': 'dmt_dq_vector_bits',
@@ -206,7 +215,7 @@ for the event.
 idq_veto = {'gstlal': False,
             'spiir': False,
             'pycbc': False,
-            'MBTAOnline': False,
+            'MBTA': False,
             'oLIB': False,
             'LIB': False,
             'CWB': False,
@@ -221,9 +230,9 @@ idq_veto = {'gstlal': False,
 Currently all False, pending iDQ review (should be done before O3).
 """
 
-low_latency_frame_types = {'H1': 'H1_O3_llhoft',
-                           'L1': 'L1_O3_llhoft',
-                           'V1': 'V1_O3_llhoft'}
+low_latency_frame_types = {'H1': 'H1_O3ReplayMDC_llhoft',
+                           'L1': 'L1_O3ReplayMDC_llhoft',
+                           'V1': 'V1_O3ReplayMDC_llhoft'}
 """Types of low latency frames used in Parameter Estimation (see
 :mod:`gwcelery.tasks.inference`) and in cache creation for detchar
 checks (see :mod:`gwcelery.tasks.detchar`).
@@ -237,9 +246,9 @@ creation for detchar checks. They do not exist for O3Replay data. (see
 :mod:`gwcelery.tasks.inference` and :mod:`gwcelery.tasks.detchar`)
 """
 
-strain_channel_names = {'H1': 'H1:GDS-CALIB_STRAIN_O3Replay',
-                        'L1': 'L1:GDS-CALIB_STRAIN_O3Replay',
-                        'V1': 'V1:Hrec_hoft_16384Hz_O3Replay'}
+strain_channel_names = {'H1': 'H1:GDS-CALIB_STRAIN_INJ1_O3Replay',
+                        'L1': 'L1:GDS-CALIB_STRAIN_INJ1_O3Replay',
+                        'V1': 'V1:Hrec_hoft_16384Hz_INJ1_O3Replay'}
 """Names of h(t) channels used in Parameter Estimation (see
 :mod:`gwcelery.tasks.inference`) and in detchar omegascan creation
 (see :mod:`gwcelery.tasks.detchar`)."""
@@ -334,5 +343,20 @@ only_alert_for_mdc = False
 """If True, then only sends alerts for MDC events. Useful for times outside
 of observing runs."""
 
+joint_mdc_freq = 2
+"""Determines how often an external MDC event will be created near an
+MDC superevent to test the RAVEN alert pipeline, i.e for every x
+MDC superevents an external MDC event is created."""
+
+bilby_default_mode = 'fast-test'
+"""Sampling mode of bilby"""
+
+condor_retry_kwargs = dict(
+    max_retries=45, retry_backoff=True, retry_jitter=False,
+    retry_backoff_max=600
+)
+"""Retry settings of condor.submit task. With these settings, a condor job is
+no longer tracked ~6 hours after it starts."""
+
 # Delete imported modules so that they do not pollute the config object
 del os, getpass
