@@ -3,15 +3,16 @@ from importlib import resources
 import io
 import random
 
-from celery.task import PeriodicTask
+from celery import group
 from celery.utils.log import get_task_logger
+from ligo.lw.table import Table
 from ligo.lw import utils
 from ligo.lw import lsctables
-import ligo.lw.table
 import lal
 from ligo.skymap.io.events.ligolw import ContentHandler
 import numpy as np
 
+from .core import get_last
 from ..data import first2years as data_first2years
 from ..import app
 from . import gracedb
@@ -33,7 +34,7 @@ def pick_coinc():
             lsctables.SimInspiralTable.tableName,
             lsctables.SummValueTable.tableName,
             lsctables.SearchSummVarsTable.tableName):
-        root.removeChild(ligo.lw.table.get_table(xmldoc, name))
+        root.removeChild(Table.get_table(xmldoc, name))
 
     coinc_inspiral_table = table = lsctables.CoincInspiralTable.get_table(
         xmldoc)
@@ -138,13 +139,24 @@ def _vet_event(superevents):
         ).apply_async()
 
 
-@gracedb.task(ignore_result=True, shared=False)
-def _upload_psd(graceid):
-    psd = resources.read_binary(data_first2years, 'psd.xml.gz')
-    gracedb.upload(psd, 'psd.xml.gz', graceid, 'Noise PSD', ['psd'])
+@app.task(shared=False)
+def _log_event_and_return_query(event):
+    graceid = event['graceid']
+    log.info('uploaded as %s', graceid)
+    return 'MDC event: {}'.format(graceid)
 
 
-@app.task(base=PeriodicTask, shared=False, run_every=3600)
+@app.on_after_finalize.connect
+def setup_periodic_tasks(sender, **kwargs):
+    """Register periodic tasks.
+
+    See
+    https://docs.celeryproject.org/en/stable/userguide/periodic-tasks.html.
+    """
+    sender.add_periodic_task(3600.0, upload_event)
+
+
+@app.task(ignore_result=True, shared=False)
 def upload_event():
     """Upload a random event from the "First Two Years" paper.
 
@@ -152,27 +164,28 @@ def upload_event():
     retraction or initial notice respectively.
     """
     coinc = pick_coinc()
-
-    graceid = gracedb.create_event(coinc, 'MDC', 'gstlal', 'CBC')
-    log.info('uploaded as %s', graceid)
-
-    if app.conf['mock_events_simulate_multiple_uploads']:
-        num = 15
-        for _ in range(num):
-            (
-                gracedb.create_event.s(
-                    _jitter_snr(coinc), 'MDC', 'gstlal', 'CBC'
-                )
-                |
-                _upload_psd.s()
-            ).apply_async()
+    num = 16 if app.conf['mock_events_simulate_multiple_uploads'] else 1
 
     (
-        _upload_psd.si(graceid)
+        group(
+            gracedb.create_event.si(
+                 _jitter_snr(coinc), 'MDC', 'gstlal', 'CBC'
+            ) for _ in range(num)
+        )
         |
-        gracedb.get_superevents.si(
-            'MDC event: {}'.format(graceid)
-        ).set(countdown=600)
+        get_last.s()
+        |
+        _log_event_and_return_query.s()
+        |
+        gracedb.get_superevents.s().set(countdown=600)
         |
         _vet_event.s()
     ).apply_async()
+
+    # Return gpstime to create an external MDC event with similar time
+    coinc_xml = io.BytesIO(coinc)
+    xmldoc = utils.load_fileobj(coinc_xml, contenthandler=ContentHandler)
+    coinc_inspiral_table = lsctables.CoincInspiralTable.get_table(xmldoc)
+    for row in coinc_inspiral_table:
+        # The first value always returns the correct gpstime
+        return float(row.end)
