@@ -10,7 +10,7 @@ from ligo import p_astro_computation as pastrocomp
 from matplotlib import pyplot as plt
 import numpy as np
 
-from . import gracedb, lvalert
+from . import gracedb, igwn_alert
 from .. import app
 from ..util import closing_figures, PromiseProxy, read_json
 
@@ -161,17 +161,19 @@ def plot(contents):
     return outfile.getvalue()
 
 
-@lvalert.handler('superevent',
-                 'mdc_superevent',
-                 shared=False)
+@igwn_alert.handler('superevent',
+                    'mdc_superevent',
+                    shared=False)
 def handle(alert):
     """LVAlert handler to plot and upload a visualization of every
-    ``p_astro.json`` that is added to a superevent.
+    ``*.p_astro.json`` file that is added to a superevent.
     """
     graceid = alert['uid']
-    filename = 'p_astro.json'
+    filename = alert['data'].get('filename')
+    p_astro_filenames = {f'{pipeline}.p_astro.json' for pipeline in
+                         ['gstlal', 'mbta', 'pycbc', 'spiir']}
 
-    if alert['alert_type'] == 'log' and alert['data']['filename'] == filename:
+    if alert['alert_type'] == 'log' and filename in p_astro_filenames:
         (
             gracedb.download.s(filename, graceid)
             |
