@@ -10,7 +10,8 @@ from . import gracedb
 log = get_task_logger(__name__)
 
 
-@app.task(shared=False)
+@app.task(queue='exttrig',
+          shared=False)
 def calculate_coincidence_far(superevent, exttrig, tl, th):
     """Compute coincidence FAR for external trigger and superevent
     coincidence by calling ligo.raven.search.calc_signif_gracedb,
@@ -33,7 +34,7 @@ def calculate_coincidence_far(superevent, exttrig, tl, th):
 
     #  Don't compute coinc FAR for SNEWS coincidences
     if exttrig['pipeline'] == 'SNEWS':
-        return
+        return {}
 
     if {'EXT_SKYMAP_READY', 'SKYMAP_READY'}.issubset(exttrig['labels']):
         #  if both sky maps available, calculate spatial coinc far
@@ -47,6 +48,8 @@ def calculate_coincidence_far(superevent, exttrig, tl, th):
                    se_dict=superevent, ext_dict=exttrig,
                    grb_search=exttrig['search'],
                    se_fitsfile=se_skymap, ext_fitsfile=ext_skymap,
+                   se_moc=True, ext_moc=False,
+                   use_radec=True if exttrig['pipeline'] == 'Swift' else False,
                    incl_sky=True, gracedb=gracedb.client,
                    far_grb=exttrig['far'])
     else:
@@ -58,9 +61,10 @@ def calculate_coincidence_far(superevent, exttrig, tl, th):
                    far_grb=exttrig['far'])
 
 
-@app.task(shared=False)
+@app.task(queue='exttrig',
+          shared=False)
 def coincidence_search(gracedb_id, alert_object, group=None, pipelines=[],
-                       searches=[]):
+                       searches=[], se_searches=[]):
     """Perform ligo-raven search for coincidences. Determines time window to
     use. If events found, launches raven pipeline.
 
@@ -69,7 +73,7 @@ def coincidence_search(gracedb_id, alert_object, group=None, pipelines=[],
     gracedb_id: str
         ID of the trigger used by GraceDB
     alert_object: dict
-        lvalert['object']
+        Alert dictionary
     group: str
         Burst or CBC
     pipelines: list
@@ -80,7 +84,7 @@ def coincidence_search(gracedb_id, alert_object, group=None, pipelines=[],
 
     (
         search.si(gracedb_id, alert_object, tl, th, group, pipelines,
-                  searches)
+                  searches, se_searches)
         |
         raven_pipeline.s(gracedb_id, alert_object, tl, th, group)
     ).delay()
@@ -131,9 +135,10 @@ def _time_window(gracedb_id, group, pipelines, searches):
     return tl, th
 
 
-@app.task(shared=False)
+@app.task(queue='exttrig',
+          shared=False)
 def search(gracedb_id, alert_object, tl=-5, th=5, group=None,
-           pipelines=[], searches=[]):
+           pipelines=[], searches=[], se_searches=[]):
     """Perform ligo-raven search for coincidences.
 
     Parameters
@@ -141,7 +146,7 @@ def search(gracedb_id, alert_object, tl=-5, th=5, group=None,
     gracedb_id: str
         ID of the trigger used by GraceDB
     alert_object: dict
-        lvalert['object']
+        Alert dictionary
     tl: int
         number of seconds to search before
     th: int
@@ -161,10 +166,12 @@ def search(gracedb_id, alert_object, tl=-5, th=5, group=None,
                                     event_dict=alert_object,
                                     gracedb=gracedb.client,
                                     group=group, pipelines=pipelines,
-                                    searches=searches)
+                                    searches=searches,
+                                    se_searches=se_searches)
 
 
-@app.task(shared=False)
+@app.task(queue='exttrig',
+          shared=False)
 def raven_pipeline(raven_search_results, gracedb_id, alert_object, tl, th,
                    gw_group):
     """Executes much of the full raven pipeline, including adding
@@ -180,41 +187,46 @@ def raven_pipeline(raven_search_results, gracedb_id, alert_object, tl, th,
     gracedb_id: str
         ID of either a superevent or external trigger
     alert_object: dict
-        lvalert['object'], either a superevent or an external event
+        Alert dictionary, either a superevent or an external event
     gw_group: str
         Burst or CBC
 
     """
     if not raven_search_results:
         return
-    if alert_object.get('group') == 'External':
+    if 'S' not in gracedb_id:
         raven_search_results = preferred_superevent(raven_search_results)
     for result in raven_search_results:
-        if alert_object.get('group') == 'External':
-            superevent_id = result['superevent_id']
-            exttrig_id = gracedb_id
-            superevent = result
-            ext_event = alert_object
-        elif 'S' in gracedb_id:
+        if 'S' in gracedb_id:
             superevent_id = gracedb_id
             exttrig_id = result['graceid']
             superevent = alert_object
             ext_event = result
+        else:
+            superevent_id = result['superevent_id']
+            exttrig_id = gracedb_id
+            superevent = result
+            ext_event = alert_object
 
         canvas = (
             gracedb.add_event_to_superevent.si(superevent_id, exttrig_id)
             |
             calculate_coincidence_far.si(superevent, ext_event, tl, th)
             |
+            update_coinc_far.s(superevent, ext_event)
+            |
             group(gracedb.create_label.si('EM_COINC', superevent_id),
                   gracedb.create_label.si('EM_COINC', exttrig_id),
                   trigger_raven_alert.s(superevent, gracedb_id,
-                                        ext_event, gw_group))
+                                        ext_event, gw_group),
+                  external_skymaps.plot_overlap_integral.s(
+                      superevent_id, exttrig_id))
         )
         canvas.delay()
 
 
-@app.task(shared=False)
+@app.task(queue='exttrig',
+          shared=False)
 def preferred_superevent(raven_search_results):
     """Chooses the superevent with the lowest far for an external
     event to be added to. This is to prevent errors from trying to
@@ -231,7 +243,60 @@ def preferred_superevent(raven_search_results):
     return [raven_search_results[idx]]
 
 
-@app.task(shared=False)
+@app.task(queue='exttrig',
+          shared=False)
+def update_coinc_far(coinc_far_dict, superevent, ext_event):
+    """Update joint info in superevent based on the current preferred
+    coincidence. This prefers a spacetime joint FAR over a time-only joint
+    FAR. A SNEWS coincidence is preferred over either.
+
+      Parameters
+    ----------
+    coinc_far_dict : dict
+        Dictionary containing coincidence false alarm rate results from
+        RAVEN
+    superevent : dict
+        superevent dictionary
+    ext_event: dict
+        external event dictionary
+
+    """
+    #  Get graceids
+    superevent_id = superevent['superevent_id']
+    ext_id = ext_event['graceid']
+
+    #  Joint FAR isn't computed for SNEWS coincidence
+    #  Choose SNEWS coincidence over any other type of coincidence
+    if ext_event['pipeline'] == 'SNEWS':
+        gracedb.update_superevent(superevent_id, em_type=ext_id,
+                                  time_coinc_far=None,
+                                  space_coinc_far=None)
+        return coinc_far_dict
+
+    #  Get the latest info to prevent race condition
+    superevent_f = gracedb.get_superevent(superevent_id)
+
+    #  Load needed variables
+    infty = float('inf')
+    new_time_far = coinc_far_dict['temporal_coinc_far']
+    new_space_far = coinc_far_dict['spatiotemporal_coinc_far']
+    #  Map None to infinity to make logic easier
+    new_space_far_f = new_space_far if new_space_far else infty
+    old_time_far = superevent_f['time_coinc_far']
+    old_time_far_f = old_time_far if old_time_far else infty
+    old_space_far = superevent_f['space_coinc_far']
+    old_space_far_f = old_space_far if old_space_far else infty
+
+    if new_space_far_f < old_space_far_f or \
+            (new_time_far < old_time_far_f and old_space_far_f == infty):
+        gracedb.update_superevent(superevent_id, em_type=ext_id,
+                                  time_coinc_far=new_time_far,
+                                  space_coinc_far=new_space_far)
+    return coinc_far_dict
+
+
+@app.task(queue='exttrig',
+          shared=False)
 def trigger_raven_alert(coinc_far_dict, superevent, gracedb_id,
                         ext_event, gw_group):
     """Determine whether an event should be published as a preliminary alert.
@@ -307,22 +372,23 @@ def trigger_raven_alert(coinc_far_dict, superevent, gracedb_id,
         coinc_far_f = coinc_far * trials_factor * (trials_factor - 1.)
         pass_far_threshold = coinc_far_f <= far_threshold
 
-    no_previous_alert = {'RAVEN_ALERT'}.isdisjoint(
-        gracedb.get_labels(superevent_id))
-    likely_real_ext_event = {'NOT_GRB'}.isdisjoint(ext_event['labels'])
+    #  Get most recent labels to prevent race conditions
+    ext_labels = gracedb.get_labels(ext_id)
+    no_previous_alert = {'RAVEN_ALERT'}.isdisjoint(ext_labels)
+    likely_real_ext_event = {'NOT_GRB'}.isdisjoint(ext_labels)
+    is_test_event = (superevent['preferred_event_data']['group'] == 'Test' or
+                     ext_event['group'] == 'Test')
 
     #  If publishable, trigger an alert by applying `RAVEN_ALERT` label to
     #  preferred event
     if pass_far_threshold and not is_ext_subthreshold and \
-            likely_real_ext_event and not missing_skymap:
-        messages.append('RAVEN: publishing criteria met for %s' % (
-            preferred_gwevent_id))
+            likely_real_ext_event and not missing_skymap and \
+            not is_test_event:
+        messages.append('RAVEN: publishing criteria met for {0}-{1}'.format(
+            preferred_gwevent_id, ext_id))
         if no_previous_alert:
-            gracedb.update_superevent(superevent_id, em_type=ext_id,
-                                      time_coinc_far=time_coinc_far,
-                                      space_coinc_far=space_coinc_far)
-            messages.append('Triggering RAVEN alert for %s' % (
-                preferred_gwevent_id))
+            messages.append('Triggering RAVEN alert for {0}-{1}'.format(
+                preferred_gwevent_id, ext_id))
             (
                 gracedb.create_label.si('RAVEN_ALERT', superevent_id)
                 |
@@ -331,26 +397,72 @@ def trigger_raven_alert(coinc_far_dict, superevent, gracedb_id,
                 gracedb.create_label.si('RAVEN_ALERT', preferred_gwevent_id)
             ).delay()
     if not pass_far_threshold:
-        messages.append(('RAVEN: publishing criteria not met for %s,'
-                         ' %s FAR (w/ trials) too large (%s>%s)' % (
-                             preferred_gwevent_id, far_type,
+        messages.append(('RAVEN: publishing criteria not met for {0}-{1},'
+                         ' {2} FAR (w/ trials) too large '
+                         '({3:.4g} > {4:.4g})'.format(
+                             preferred_gwevent_id, ext_id, far_type,
                              coinc_far_f, far_threshold)))
     if is_ext_subthreshold:
-        messages.append(('RAVEN: publishing criteria not met for %s,'
-                         ' %s is subthreshold' % (preferred_gwevent_id,
-                                                  ext_id)))
+        messages.append(('RAVEN: publishing criteria not met for {0}-{1},'
+                         ' {1} is subthreshold'.format(preferred_gwevent_id,
+                                                       ext_id)))
     if not likely_real_ext_event:
-        messages.append(('RAVEN: %s is likely non-astrophysical.'
-                         % (ext_id)))
+        messages.append(('RAVEN: publishing criteria not met for {0}-{1},'
+                         ' {1} is likely non-astrophysical.'.format(
+                             preferred_gwevent_id, ext_id)))
+    if is_test_event:
+        messages.append('RAVEN: {0}-{1} is non-astrophysical, '
+                        'at least one event is a Test event'.format(
+                            preferred_gwevent_id, ext_id))
     if not no_previous_alert:
-        messages.append(('RAVEN: Alert already triggered for  %s'
-                         % (superevent_id)))
+        messages.append(('RAVEN: Alert already triggered for {}'.format(
+                            ext_id)))
     if missing_skymap:
         messages.append('RAVEN: Will only publish Swift coincidence '
                         'event if spatial-temporal FAR is present. '
                         'Waiting for both sky maps to be available '
                         'first.')
     for message in messages:
-        gracedb.upload(None, None, superevent_id,
-                       message,
-                       tags=['ext_coinc'])
+        gracedb.upload.si(None, None, superevent_id, message,
+                          tags=['ext_coinc']).delay()
+
+
+@app.task(queue='exttrig',
+          shared=False)
+def sog_paper_pipeline(ext_event, superevent):
+    """Determine whether an a speed of gravity measurment manuscript should be
+    triggered for a given coincidence.
+    This is denoted by applying the SOG_READY label to a superevent.
+
+    All of the following conditions must be true for a SoG paper:
+
+    *   The coincidence is significant and FARs more significant than in
+        :obj:`~sog_paper_far_threshold`.
+    *   The external event is a high-significance GRB and from a MOU partner.
+    *   The GW event is a CBC candidate.
+
+    Parameters
+    ----------
+    superevent : dict
+        superevent dictionary
+    ext_event: dict
+        external event dictionary
+
+    """
+    gw_far = superevent['far']
+    coinc_far = superevent['space_coinc_far']
+    gw_far_threshold = app.conf['sog_paper_far_threshold']['gw']
+    joint_far_threshold = app.conf['sog_paper_far_threshold']['joint']
+
+    #  Check publishing conditions
+    pass_gw_far_threshold = gw_far <= gw_far_threshold
+    pass_joint_far_threshold = coinc_far <= joint_far_threshold
+    is_grb = ext_event['search'] in ['GRB', 'MDC']
+    is_mou_partner = ext_event['pipeline'] in ['Fermi', 'Swift']
+    is_cbc = superevent['preferred_event_data']['group'] == 'CBC'
+
+    if is_grb and is_cbc and is_mou_partner and \
+            pass_gw_far_threshold and pass_joint_far_threshold:
+        #  Trigger SOG_READY label alert to alert SOG analysts
+        gracedb.create_label.si('SOG_READY',
+                                superevent['superevent_id']).delay()
