@@ -8,6 +8,7 @@ from astropy import table
 from celery import group
 from celery.exceptions import Ignore
 from ligo.skymap.tool import ligo_skymap_flatten
+from ligo.skymap.tool import ligo_skymap_unflatten
 from ligo.skymap.tool import ligo_skymap_from_samples
 from ligo.skymap.tool import ligo_skymap_plot
 from ligo.skymap.tool import ligo_skymap_plot_volume
@@ -15,7 +16,7 @@ from matplotlib import pyplot as plt
 import numpy as np
 
 from . import gracedb
-from . import lvalert
+from . import igwn_alert
 from ..import app
 from ..jinja import env
 from ..util.cmdline import handling_system_exit
@@ -30,7 +31,18 @@ def annotate_fits(filecontents, versioned_filename, graceid, tags):
     This function downloads a FITS file and then generates and uploads all
     derived images as well as an HTML dump of the FITS header.
     """
-    filebase = versioned_filename.partition('.fits')[0]
+    multiorder_extension = '.multiorder.fits'
+    flat_extension = '.fits'
+
+    if multiorder_extension in versioned_filename:
+        extension = multiorder_extension
+        multiorder = True
+    else:
+        extension = flat_extension
+        multiorder = False
+
+    filebase, _, _ = versioned_filename.partition(extension)
+
     header_msg = (
         'FITS headers for <a href="/api/superevents/{graceid}/files/'
         '{versioned_filename}">{versioned_filename}</a>').format(
@@ -43,18 +55,34 @@ def annotate_fits(filecontents, versioned_filename, graceid, tags):
         'Volume rendering of <a href="/api/superevents/{graceid}/files/'
         '{versioned_filename}">{versioned_filename}</a>').format(
             graceid=graceid, versioned_filename=versioned_filename)
+    flatten_msg = (
+        'Flat-resolution fits file created from '
+        '<a href="/api/superevents/{graceid}/files/'
+        '{versioned_filename}">{versioned_filename}</a>').format(
+            graceid=graceid, versioned_filename=versioned_filename)
 
     group(
-        fits_header.s(versioned_filename) |
+        fits_header.s(versioned_filename)
+        |
         gracedb.upload.s(
             filebase + '.html', graceid, header_msg, tags),
 
-        plot_allsky.s() |
+        plot_allsky.s()
+        |
         gracedb.upload.s(
             filebase + '.png', graceid, allsky_msg, tags),
 
         annotate_fits_volume.s(
-            filebase + '.volume.png', graceid, volume_msg, tags)
+            filebase + '.volume.png', graceid, volume_msg, tags),
+
+        *(
+            [
+                flatten.s(f'{filebase}.fits.gz')
+                |
+                gracedb.upload.s(
+                    f'{filebase}.fits.gz', graceid, flatten_msg, tags)
+            ] if multiorder else []
+        )
     ).delay(filecontents)
 
 
@@ -136,6 +164,20 @@ def flatten(filecontents, filename):
         return open(outfilename, 'rb').read()
 
 
+@app.task(shared=False)
+def unflatten(filecontents, filename):
+    """Convert a HEALPix FITS file to multi-resolution UNIQ indexing from
+    the more common IMPLICIT indexing using the command-line tool
+    :doc:`ligo-skymap-unflatten <ligo.skymap:tool/ligo_skymap_unflatten>`.
+    """
+    with NamedTemporaryFile(content=filecontents) as infile, \
+            tempfile.TemporaryDirectory() as tmpdir, \
+            handling_system_exit():
+        outfilename = os.path.join(tmpdir, filename)
+        ligo_skymap_unflatten.main([infile.name, outfilename])
+        return open(outfilename, 'rb').read()
+
+
 @app.task(shared=False, queue='openmp')
 def skymap_from_samples(samplefilecontents):
     """Generate multi-resolution fits file from samples."""
@@ -151,7 +193,8 @@ def skymap_from_samples(samplefilecontents):
 def plot_bayes_factor(logb,
                       values=(1, 3, 5),
                       labels=('', 'strong', 'very strong'),
-                      xlim=7, title=None, palette='RdYlBu'):
+                      xlim=7, title=None, palette='RdYlBu',
+                      var_label="B"):
     """Visualize a Bayes factor as a `bullet graph`_.
 
     Make a bar chart of a log Bayes factor as compared to a set of subjective
@@ -174,6 +217,8 @@ def plot_bayes_factor(logb,
         Title for plot.
     palette : str
         Color palette.
+    var_label : str
+        The variable symbol used in plotting
 
     Returns
     -------
@@ -195,7 +240,7 @@ def plot_bayes_factor(logb,
         ax.set_ylim(-0.5, 0.5)
         ax.set_yticks([])
         ax.set_title(title)
-        ax.set_ylabel(r'$\ln\,B$', rotation=0,
+        ax.set_ylabel(r'$\ln\,{}$'.format(var_label), rotation=0,
                       rotation_mode='anchor',
                       ha='right', va='center')
 
@@ -238,7 +283,7 @@ def plot_bayes_factor(logb,
 @app.task(shared=False)
 @closing_figures()
 def plot_coherence(filecontents):
-    """LVAlert handler to plot the coherence Bayes factor.
+    """IGWN alert handler to plot the coherence Bayes factor.
 
     Parameters
     ----------
@@ -274,11 +319,11 @@ def plot_coherence(filecontents):
     return outfile.getvalue()
 
 
-@lvalert.handler('superevent',
-                 'mdc_superevent',
-                 shared=False)
+@igwn_alert.handler('superevent',
+                    'mdc_superevent',
+                    shared=False)
 def handle_plot_coherence(alert):
-    """LVAlert handler to plot and upload a visualization of the coherence
+    """IGWN alert handler to plot and upload a visualization of the coherence
     Bayes factor.
 
     Notes
