@@ -12,7 +12,7 @@ from celery.utils.log import get_task_logger
 from ligo.segments import segment, segmentlist
 
 from ..import app
-from . import gracedb, lvalert
+from . import gracedb, igwn_alert
 
 log = get_task_logger(__name__)
 
@@ -28,21 +28,25 @@ FROZEN_LABEL = 'EM_Selected'
 """This label indicates that the superevent manager should make no further
 changes to the preferred event."""
 
+EARLY_WARNING_LABEL = 'EARLY_WARNING'
+"""This label indicates that the superevent contains a significant
+early warning event."""
+
 READY_LABEL = 'EM_READY'
 """This label indicates that a preferred event has been assigned and it
 has all data products required to make it ready for annotations."""
 
 
-@lvalert.handler('cbc_gstlal',
-                 'cbc_spiir',
-                 'cbc_pycbc',
-                 'cbc_mbtaonline',
-                 'burst_olib',
-                 'burst_cwb',
-                 shared=False)
+@igwn_alert.handler('cbc_gstlal',
+                    'cbc_spiir',
+                    'cbc_pycbc',
+                    'cbc_mbta',
+                    'burst_olib',
+                    'burst_cwb',
+                    shared=False)
 def handle(payload):
-    """Respond to lvalert nodes from low-latency search pipelines and delegate
-    to :meth:`process` for superevent management.
+    """Respond to IGWN alert topics from low-latency search pipelines and
+    delegate to :meth:`process` for superevent management.
     """
     alert_type = payload['alert_type']
     gid = payload['object']['graceid']
@@ -86,7 +90,7 @@ def process(payload):
     Parameters
     ----------
     payload : dict
-        LVAlert payload
+        IGWN alert payload
 
     """
     event_info = payload['object']
@@ -97,24 +101,23 @@ def process(payload):
     if event_info.get('superevent'):
         sid = event_info['superevent']
         log.info('Event %s already belongs to superevent %s', gid, sid)
-        s = gracedb.get_superevent(sid)
+        # superevent_neighbours has current and nearby superevents
+        s = event_info['superevent_neighbours'][sid]
         superevent = _SuperEvent(s['t_start'],
                                  s['t_end'],
                                  s['t_0'],
-                                 s['superevent_id'],
-                                 s['preferred_event'], s)
-        _update_superevent(superevent,
+                                 s['superevent_id'])
+        _update_superevent(superevent.superevent_id,
                            event_info,
                            t_0=t_0,
                            t_start=None,
                            t_end=None)
-    else:  # not event_info.get('superevent')
+    else:
         log.info('Event %s does not yet belong to a superevent', gid)
         superevents = gracedb.get_superevents('category: {} {} .. {}'.format(
             category,
             event_info['gpstime'] - app.conf['superevent_query_d_t_start'],
             event_info['gpstime'] + app.conf['superevent_query_d_t_end']))
-
         for s in superevents:
             if gid in s['gw_events']:
                 sid = s['superevent_id']
@@ -123,30 +126,25 @@ def process(payload):
                     log.info('Label %s added to %s',
                              payload['data']['name'], gid)
                 elif payload['alert_type'] == 'new':
-                    log.info('New type LVAlert for %s with '
+                    log.info('new alert type for %s with '
                              'existing superevent %s. '
                              'No action required', gid, sid)
                     return
                 superevent = _SuperEvent(s['t_start'],
                                          s['t_end'],
                                          s['t_0'],
-                                         s['superevent_id'],
-                                         s['preferred_event'], s)
-                _update_superevent(superevent,
+                                         s['superevent_id'])
+                _update_superevent(superevent.superevent_id,
                                    event_info,
                                    t_0=t_0,
                                    t_start=None,
                                    t_end=None)
                 break
         else:  # s not in superevents
-            event_segment = _Event(t_0, t_start, t_end,
-                                   event_info['graceid'],
-                                   event_info['group'],
-                                   event_info['pipeline'],
-                                   event_info.get('search'),
-                                   event_dict=event_info)
+            event_segment = _Event(t_start, t_end, t_0, event_info['graceid'])
 
-            superevent = _partially_intersects(superevents, event_segment)
+            superevent = _partially_intersects(superevents,
+                                               event_segment)
 
             if superevent:
                 sid = superevent.superevent_id
@@ -165,7 +163,7 @@ def process(payload):
                     log.info('%s is completely contained in %s',
                              event_segment.gid, sid)
                     new_t_start = new_t_end = None
-                _update_superevent(superevent,
+                _update_superevent(superevent.superevent_id,
                                    event_info,
                                    t_0=t_0,
                                    t_start=new_t_start,
@@ -180,7 +178,7 @@ def process(payload):
         gracedb.create_label.delay('ADVREQ', sid)
         if is_complete(event_info):
             if app.conf['preliminary_alert_timeout'] \
-                    and 'EARLY_WARNING' not in event_info['labels']:
+                    and EARLY_WARNING_LABEL not in event_info['labels']:
                 gracedb.create_label.s(FROZEN_LABEL, sid).set(
                     queue='superevent',
                     countdown=app.conf['preliminary_alert_timeout']
@@ -225,7 +223,8 @@ def get_ts(event):
     ----------
     event : dict
         Event dictionary (e.g., the return value from
-        :meth:`gwcelery.tasks.gracedb.get_event`).
+        :meth:`gwcelery.tasks.gracedb.get_event` or
+        ``preferred_event_data`` in igwn-alert packet.)
 
     Returns
     -------
@@ -264,7 +263,8 @@ def get_snr(event):
     ----------
     event : dict
         Event dictionary (e.g., the return value from
-        :meth:`gwcelery.tasks.gracedb.get_event`).
+        :meth:`gwcelery.tasks.gracedb.get_event`, or
+        ``preferred_event_data`` in igwn-alert packet.)
 
     Returns
     -------
@@ -294,7 +294,8 @@ def get_instruments(event):
     ----------
     event : dict
         Event dictionary (e.g., the return value from
-        :meth:`gwcelery.tasks.gracedb.get_event`).
+        :meth:`gwcelery.tasks.gracedb.get_event`, or
+        ``preferred_event_data`` in igwn-alert packet.)
 
     Returns
     -------
@@ -314,7 +315,8 @@ def get_instruments_in_ranking_statistic(event):
     ----------
     event : dict
         Event dictionary (e.g., the return value from
-        :meth:`gwcelery.tasks.gracedb.get_event`).
+        :meth:`gwcelery.tasks.gracedb.get_event`, or
+        ``preferred_event_data`` in igwn-alert packet.)
 
     Returns
     -------
@@ -376,7 +378,8 @@ def is_complete(event):
     ----------
     event : dict
         Event dictionary (e.g., the return value from
-        :meth:`gwcelery.tasks.gracedb.get_event`).
+        :meth:`gwcelery.tasks.gracedb.get_event`, or
+        ``preferred_event_data`` in igwn-alert packet.)
 
     """
     group = event['group'].lower()
@@ -401,7 +404,8 @@ def should_publish(event):
     ----------
     event : dict
         Event dictionary (e.g., the return value from
-        :meth:`gwcelery.tasks.gracedb.get_event`).
+        :meth:`gwcelery.tasks.gracedb.get_event`, or
+        ``preferred_event_data`` in igwn-alert packet.)
 
     Returns
     -------
@@ -418,7 +422,7 @@ def _should_publish(event):
     the publishability criteria as a tuple for later use.
     """
     group = event['group'].lower()
-    if 'EARLY_WARNING' in event['labels']:
+    if EARLY_WARNING_LABEL in event['labels']:
         far_threshold = app.conf['early_warning_alert_far_threshold']
         trials_factor = app.conf['early_warning_alert_trials_factor']
     else:
@@ -476,7 +480,7 @@ def keyfunc(event):
             significance)
 
 
-def _update_superevent(superevent, new_event_dict,
+def _update_superevent(superevent_id, new_event_dict,
                        t_0, t_start, t_end):
     """Update preferred event and/or change time window. Events with multiple
     detectors take precedence over single-detector events, then CBC events take
@@ -486,8 +490,8 @@ def _update_superevent(superevent, new_event_dict,
 
     Parameters
     ----------
-    superevent : object
-        instance of :class:`_SuperEvent`
+    superevent_id : str
+        the superevent_id
     new_event_dict : dict
         event info of the new trigger as a dictionary
     t_0 : float
@@ -498,16 +502,17 @@ def _update_superevent(superevent, new_event_dict,
         end time of `superevent_id`, None for no change
 
     """
-    superevent_id = superevent.superevent_id
-    preferred_event = superevent.preferred_event
+    # labels and preferred event in the IGWN alert are not the latest
+    superevent_dict = gracedb.get_superevent(superevent_id)
 
+    superevent_labels = superevent_dict['labels']
+    preferred_event_dict = superevent_dict['preferred_event_data']
     kwargs = {}
     if t_start is not None:
         kwargs['t_start'] = t_start
     if t_end is not None:
         kwargs['t_end'] = t_end
-    if FROZEN_LABEL not in superevent.event_dict['labels']:
-        preferred_event_dict = gracedb.get_event(preferred_event)
+    if FROZEN_LABEL not in superevent_labels:
         if keyfunc(new_event_dict) > keyfunc(preferred_event_dict):
             # update preferred event when EM_Selected is not applied
             kwargs['t_0'] = t_0
@@ -515,6 +520,7 @@ def _update_superevent(superevent, new_event_dict,
 
     if kwargs:
         gracedb.update_superevent(superevent_id, **kwargs)
+
     # completeness takes first precedence in deciding preferred event
     # necessary and suffiecient condition to superevent as ready
     if is_complete(new_event_dict):
@@ -528,8 +534,8 @@ def _superevent_segment_list(superevents):
     Parameters
     ----------
     superevents : list
-        List of superevent dictionaries (e.g., the return value from
-        :meth:`gwcelery.tasks.gracedb.get_superevents`).
+        List of superevent dictionaries (e.g., the values
+        of field ``superevent_neighbours`` in igwn-alert packet).
 
     Returns
     -------
@@ -537,13 +543,9 @@ def _superevent_segment_list(superevents):
         superevents as a segmentlist object
 
     """
-    return segmentlist([_SuperEvent(s.get('t_start'),
-                        s.get('t_end'),
-                        s.get('t_0'),
-                        s.get('superevent_id'),
-                        s.get('preferred_event'),
-                        s)
-                       for s in superevents])
+    return segmentlist(
+        [_SuperEvent(s['t_start'], s['t_end'], s['t_0'], s['superevent_id'])
+         for s in superevents])
 
 
 def _partially_intersects(superevents, event_segment):
@@ -554,8 +556,8 @@ def _partially_intersects(superevents, event_segment):
     Parameters
     ----------
     superevents : list
-        list pulled down using the gracedb client
-        :method:`superevents`
+        list of superevents. Typical value of
+        ``superevent_neighbours.values()``.
     event_segment : segment
         segment object whose index is wanted
 
@@ -576,17 +578,12 @@ def _partially_intersects(superevents, event_segment):
 class _Event(segment):
     """An event implemented as an extension of :class:`segment`."""
 
-    def __new__(cls, t0, t_start, t_end, *args, **kwargs):
+    def __new__(cls, t_start, t_end, *args, **kwargs):
         return super().__new__(cls, t_start, t_end)
 
-    def __init__(self, t0, t_start, t_end, gid, group=None, pipeline=None,
-                 search=None, event_dict={}):
-        self.t0 = t0
+    def __init__(self, t_start, t_end, t_0, gid):
+        self.t_0 = t_0
         self.gid = gid
-        self.group = group
-        self.pipeline = pipeline
-        self.search = search
-        self.event_dict = event_dict
 
 
 class _SuperEvent(segment):
@@ -595,11 +592,8 @@ class _SuperEvent(segment):
     def __new__(cls, t_start, t_end, *args, **kwargs):
         return super().__new__(cls, t_start, t_end)
 
-    def __init__(self, t_start, t_end, t_0, sid,
-                 preferred_event=None, event_dict={}):
+    def __init__(self, t_start, t_end, t_0, sid):
         self.t_start = t_start
         self.t_end = t_end
         self.t_0 = t_0
         self.superevent_id = sid
-        self.preferred_event = preferred_event
-        self.event_dict = event_dict
