@@ -2,8 +2,8 @@ import configparser
 import os
 import subprocess
 
+import json
 import pytest
-from requests.exceptions import HTTPError
 from unittest.mock import Mock
 
 from .. import app
@@ -74,8 +74,8 @@ def test_find_appropriate_cal_env(tmp_path):
 
 
 @pytest.mark.parametrize(
-    'mc,q,sid,answers',
-    [[1., 1., None,
+    'mc,q,answers',
+    [[1., 1.,
       [('analysis', 'engine', 'lalinferencenest'),
        ('analysis', 'nparallel', '4'),
        ('analysis', 'roq', 'True'),
@@ -87,31 +87,28 @@ def test_find_appropriate_cal_env(tmp_path):
        ('condor', 'bayeswave_request_memory', '16000'),
        ('condor', 'bayeswavepost_request_memory', '16000'),
        ('bayeswave', 'bw_srate', '4096')]],
-     [10., 1., None,
+     [10., 1.,
       [('condor', 'bayeswave_request_memory', '1000'),
        ('condor', 'bayeswavepost_request_memory', '4000')]],
-     [10., 1., 'S1234', []],
-     [1., 1. / 10., None,
+     [1., 1. / 10.,
       [('analysis', 'engine', 'lalinferencenest'),
        ('analysis', 'nparallel', '4'),
        ('analysis', 'roq', 'True'),
        ('paths', 'roq_b_matrix_directory',
         '/home/cbc/ROQ_data/SEOBNRv4ROQ/'),
        ('engine', 'approx', 'SEOBNRv4_ROMpseudoFourPN')]],
-     [1., 1. / 20., None,
+     [1., 1. / 20.,
       [('analysis', 'engine', 'lalinferencemcmc'),
        ('analysis', 'nparallel', '10'),
        ('analysis', 'roq', 'False'),
        ('engine', 'approx', 'SEOBNRv4_ROMpseudoFourPN'),
        ('engine', 'neff', '300')]]]
 )
-def test_prepare_ini(monkeypatch, mc, q, sid, answers):
-    upload = Mock()
+def test_prepare_lalinference_ini(monkeypatch, mc, q, answers):
 
     def mock_calenv(trigtime, path):
         return 'path_to_calenv'
 
-    monkeypatch.setattr('gwcelery.tasks.gracedb.upload.run', upload)
     monkeypatch.setattr(
         'gwcelery.tasks.inference._find_appropriate_cal_env',
         mock_calenv)
@@ -127,146 +124,207 @@ def test_prepare_ini(monkeypatch, mc, q, sid, answers):
     }
     config = configparser.ConfigParser()
     config.read_string(
-        inference.prepare_ini(app.conf['low_latency_frame_types'], event, sid))
+        inference.prepare_lalinference_ini(
+            app.conf['low_latency_frame_types'], event, 'S1234'))
     for section, key, answer in answers:
         assert config[section][key] == answer
 
-    if sid is not None:
-        upload.assert_called_once()
 
-
-def test_pre_pe_tasks(monkeypatch):
-    event, sid = {'gpstime': 1187008882}, 'S1234'
-    frametype_dict = app.conf['low_latency_frame_types']
-
-    def mock_query_data(gpstime):
-        return frametype_dict
-
-    def mock_prepare_ini(f, e, s):
-        assert f == frametype_dict
-        assert e == event
-        assert s == sid
-
-    query_data = Mock(side_effect=mock_query_data)
-    prepare_ini = Mock(side_effect=mock_prepare_ini)
-    monkeypatch.setattr('gwcelery.tasks.inference.query_data.run', query_data)
-    monkeypatch.setattr('gwcelery.tasks.inference.prepare_ini.run',
-                        prepare_ini)
-
-    inference.pre_pe_tasks(event, sid).delay()
-    query_data.assert_called_once()
-    prepare_ini.assert_called_once()
-
-
-@pytest.mark.parametrize('psd', [b'psd', None])
-def test_setup_dag_for_lalinference_(monkeypatch, tmp_path, psd):
-    ini, coinc = 'ini', b'coinc'
+def test_setup_dag_for_lalinference(monkeypatch, tmp_path):
+    coinc = b'coinc'
     rundir = str(tmp_path)
+    ini = 'ini'
     dag = 'lalinference dag'
 
     def _subprocess_run(cmd, **kwargs):
-        assert cmd[0] == 'lalinference_pipe'
-        del cmd[0]
-        assert '--run-path' in cmd and '--coinc' in cmd
-        idx = cmd.index('--run-path')
-        assert cmd[idx + 1] == rundir
-        del cmd[idx:idx + 2]
-        idx = cmd.index('--coinc')
-        path_to_coinc = cmd[idx + 1]
+        assert cmd[2] == rundir
+
+        path_to_coinc = cmd[4]
+        assert os.path.exists(path_to_coinc)
         with open(path_to_coinc, 'rb') as f:
             assert f.read() == coinc
-        del cmd[idx:idx + 2]
-        if psd is not None:
-            assert '--psd' in cmd
-            idx = cmd.index('--psd')
-            path_to_psd = cmd[idx + 1]
-            with open(path_to_psd, 'rb') as f:
-                assert f.read() == psd
-            del cmd[idx:idx + 2]
-        path_to_ini, = cmd
+
+        path_to_ini = cmd[5]
+        assert os.path.exists(path_to_ini)
+        with open(path_to_ini, 'r') as f:
+            assert f.read() == ini
+
+        path_to_psd = cmd[7]
+        assert os.path.exists(path_to_psd)
+        with open(path_to_psd, 'rb') as f:
+            assert f.read() == coinc
+
         with open(os.path.join(rundir, 'multidag.dag'), 'w') as f:
             f.write(dag)
 
-    monkeypatch.setattr('subprocess.run', _subprocess_run)
-    path_to_dag = inference._setup_dag_for_lalinference(
-        (coinc, psd), ini, rundir, 'S1234')
-    with open(path_to_dag, 'r') as f:
-        assert f.read() == dag
-
-
-@pytest.mark.parametrize(
-    'host', ['gracedb-playground.ligo.org', 'gracedb.ligo.org'])
-def test_setup_dag_for_bilby(monkeypatch, tmp_path, host):
-    event, rundir = {}, str(tmp_path)
-    pid, sid = 'G1234', 'S1234'
-    monkeypatch.setitem(app.conf, 'gracedb_host', host)
-    dag = 'bilby dag'
-
-    def _subprocess_run(cmd, **kwargs):
-        is_quick_conf = 'o2replay' in cmd and 'FastTest' in cmd
-        if host == 'gracedb.ligo.org':
-            assert not is_quick_conf
-        else:
-            assert is_quick_conf
-        with open(os.path.join(rundir, 'bilby_config.ini'), 'w') as f:
-            f.write('bilby configuration ini file')
-        dir = os.path.join(rundir, 'submit')
-        os.mkdir(dir)
-        with open(os.path.join(dir, 'dag_bilby.submit'), 'w') as f:
-            f.write(dag)
-
     upload = Mock()
     monkeypatch.setattr('subprocess.run', _subprocess_run)
+    monkeypatch.setattr(
+        'gwcelery.tasks.inference.prepare_lalinference_ini',
+        Mock(return_value=ini))
     monkeypatch.setattr('gwcelery.tasks.gracedb.upload.run', upload)
 
-    path_to_dag = inference._setup_dag_for_bilby(event, rundir, pid, sid)
+    path_to_dag = inference._setup_dag_for_lalinference(
+        coinc, rundir, {}, 'S1234', {})
+
+    assert os.path.exists(path_to_dag)
     with open(path_to_dag, 'r') as f:
         assert f.read() == dag
     upload.assert_called_once()
 
 
-@pytest.mark.parametrize('pipeline', ['lalinference', 'bilby'])
+@pytest.mark.parametrize(
+    'mode', ['production', 'fast_bns', 'fast_test'])
+def test_setup_dag_for_bilby(monkeypatch, tmp_path, mode):
+    psd = b'psd'
+    rundir = str(tmp_path)
+    event = {'gpstime': 1187008882, 'graceid': 'G1234'}
+    sid = 'S1234'
+    ini = 'bilby configuration ini file'
+    dag = 'bilby dag'
+
+    def _subprocess_run(cmd, **kwargs):
+        if mode == 'fast_test':
+            assert 'o3replay' in cmd and 'FastTest' in cmd
+
+        assert cmd[2] == os.path.join(
+            app.conf['pe_results_path'], sid, 'bilby')
+
+        assert cmd[4] == rundir
+
+        path_to_json = cmd[6]
+        assert os.path.exists(path_to_json)
+        with open(path_to_json, 'r') as f:
+            assert event == json.load(f)
+
+        path_to_psd = cmd[8]
+        assert os.path.exists(path_to_psd)
+        with open(path_to_psd, 'rb') as f:
+            assert f.read() == psd
+
+        path_to_settings = cmd[10]
+        assert os.path.exists(path_to_settings)
+        ans = {
+            'summarypages_arguments': {'gracedb': event['graceid'],
+                                       'no_ligo_skymap': True},
+            'queue': 'Online_PE'
+        }
+        if mode == 'fast_bns':
+            ans.update(
+                {'sampler_kwargs': {'nact': 3, 'nlive': 500, 'npool': 24},
+                 'n_parallel': 2,
+                 'request_cpus': 24,
+                 'request_memory_generation': 8.0}
+            )
+        with open(path_to_settings, 'r') as f:
+            assert json.load(f) == ans
+
+        with open(os.path.join(rundir, 'bilby_config.ini'), 'w') as f:
+            f.write(ini)
+        dir = os.path.join(rundir, 'submit')
+        os.mkdir(dir)
+        with open(os.path.join(dir, 'dag_bilby.submit'), 'w') as f:
+            f.write(dag)
+
+    def _upload(filecontents, filename, graceid, message, tags):
+        assert filecontents == ini
+
+    upload = Mock(side_effect=_upload)
+    monkeypatch.setattr('subprocess.run', _subprocess_run)
+    monkeypatch.setattr('gwcelery.tasks.gracedb.upload.run', upload)
+
+    path_to_dag = inference._setup_dag_for_bilby(psd, rundir, event, sid, mode)
+
+    assert os.path.exists(path_to_dag)
+    with open(path_to_dag, 'r') as f:
+        assert f.read() == dag
+    upload.assert_called_once()
+
+
+def test_setup_dag_for_rapidpe(monkeypatch, tmp_path):
+    rundir = str(tmp_path)
+    dag_filename = 'event_all_iterations.dag'
+    dag_content = 'rapidpe dag'
+
+    def _subprocess_run(cmd, **kwargs):
+        path_to_ini = cmd[1]
+        assert os.path.exists(path_to_ini)
+        with open(os.path.join(rundir, dag_filename), 'w') as f:
+            f.write(dag_content)
+
+    upload = Mock()
+    monkeypatch.setattr('subprocess.run', _subprocess_run)
+    monkeypatch.setattr('gwcelery.tasks.gracedb.upload.run', upload)
+
+    path_to_dag = inference._setup_dag_for_rapidpe(rundir, 'S1234', {})
+    with open(path_to_dag, 'r') as f:
+        assert f.read() == dag_content
+    upload.assert_called_once()
+
+
+@pytest.mark.parametrize('pipeline', ['lalinference', 'bilby', 'rapidpe'])
 def test_setup_dag_for_failure(monkeypatch, tmp_path, pipeline):
     upload = Mock()
     monkeypatch.setattr('gwcelery.tasks.gracedb.upload.run', upload)
     monkeypatch.setattr('subprocess.run',
                         Mock(side_effect=subprocess.CalledProcessError(
                             1, ['cmd'], output=b'out', stderr=b'err')))
+    monkeypatch.setattr(
+        'gwcelery.tasks.inference._find_appropriate_cal_env',
+        Mock(return_value='./cal_file.txt'))
 
     rundir = str(tmp_path)
-
+    event = {
+        'gpstime': 1187008882,
+        'graceid': 'G1234',
+        'extra_attributes': {'SingleInspiral':
+                             [{'mass1': 1.4, 'mass2': 1.4, 'mchirp': 1.2}]}
+    }
     with pytest.raises(subprocess.CalledProcessError):
         if pipeline == 'lalinference':
             inference._setup_dag_for_lalinference(
-                (b'coinc', b'psd'), 'ini', rundir, 'S1234')
+                b'coinc', rundir, event, 'S1234', {})
         elif pipeline == 'bilby':
-            inference._setup_dag_for_bilby({}, rundir, 'G1234', 'S1234')
+            inference._setup_dag_for_bilby(
+                (b'psd'), rundir, event, 'S1234', 'production')
+        elif pipeline == 'rapidpe':
+            inference._setup_dag_for_rapidpe(rundir, 'S1234', {})
     assert not os.path.exists(rundir)
-    upload.assert_called_once()
+    if pipeline == 'bilby':
+        assert upload.call_count == 1
+    else:
+        # For pipelines except for bilby, an ini file is uploaded before dag
+        # generation, and hence the call count is 2.
+        assert upload.call_count == 2
 
 
-@pytest.mark.parametrize('pipeline',
-                         ['lalinference', 'bilby', 'my_awesome_pipeline'])
+@pytest.mark.parametrize(
+    'pipeline', ['lalinference', 'bilby', 'rapidpe', 'my_awesome_pipeline'])
 def test_dag_prepare_task(monkeypatch, pipeline):
-    gid, sid = 'G1234', 'S1234'
-    coinc, psd, ini, event = b'coinc', b'psd', 'ini', {'gpstime': 1187008882}
+    sid = 'S1234'
+    coinc = b'coinc'
+    event = {'gpstime': 1187008882, 'graceid': 'G1234'}
     rundir = 'rundir'
+    frametype_dict = {'H1': 'H1_llhoft', 'L1': 'L1_llhoft'}
     path_to_dag = os.path.join(rundir, 'parameter_estimation.dag')
+    kwargs = {'bilby_mode': 'production'}
 
     def mock_download(filename, gid):
         if filename == 'coinc.xml':
             return coinc
-        elif filename == 'psd.xml.gz':
-            return psd
 
-    def _setup_dag_for_lalinference(c_p, i, r, s):
-        c, p = c_p
-        assert (c == coinc and p == psd and i == ini
-                and r == rundir and s == sid)
+    def _setup_dag_for_lalinference(c, r, e, s, f):
+        assert (c == coinc and r == rundir and e == event and s == sid and
+                f == frametype_dict)
         return path_to_dag
 
-    def _setup_dag_for_bilby(e, r, p, s):
-        assert e == event and r == rundir and p == gid and s == sid
+    def _setup_dag_for_bilby(c, r, e, s, m):
+        assert c == coinc and r == rundir and e == event and s == sid and \
+            m == kwargs['bilby_mode']
+        return path_to_dag
+
+    def _setup_dag_for_rapidpe(r, s, f):
+        assert r == rundir and s == sid and f == frametype_dict
         return path_to_dag
 
     def _subprocess_run(cmd, **kwargs):
@@ -275,6 +333,7 @@ def test_dag_prepare_task(monkeypatch, pipeline):
     mock_setup_dag_for_lalinference = \
         Mock(side_effect=_setup_dag_for_lalinference)
     mock_setup_dag_for_bilby = Mock(side_effect=_setup_dag_for_bilby)
+    mock_setup_dag_for_rapidpe = Mock(side_effect=_setup_dag_for_rapidpe)
     monkeypatch.setattr('gwcelery.tasks.gracedb.download.run', mock_download)
     monkeypatch.setattr('gwcelery.tasks.gracedb.get_event.run',
                         Mock(return_value=event))
@@ -283,20 +342,27 @@ def test_dag_prepare_task(monkeypatch, pipeline):
         mock_setup_dag_for_lalinference)
     monkeypatch.setattr('gwcelery.tasks.inference._setup_dag_for_bilby.run',
                         mock_setup_dag_for_bilby)
+    monkeypatch.setattr('gwcelery.tasks.inference._setup_dag_for_rapidpe.run',
+                        mock_setup_dag_for_rapidpe)
     monkeypatch.setattr('subprocess.run', _subprocess_run)
-    if pipeline in ['lalinference', 'bilby']:
-        inference.dag_prepare_task(rundir, sid, gid, pipeline, ini).delay()
+    if pipeline in ['lalinference', 'bilby', 'rapidpe']:
+        inference.dag_prepare_task(
+            rundir, event, sid, pipeline, frametype_dict, **kwargs).delay()
         if pipeline == 'lalinference':
             mock_setup_dag_for_lalinference.assert_called_once()
         elif pipeline == 'bilby':
             mock_setup_dag_for_bilby.assert_called_once()
+        elif pipeline == 'rapidpe':
+            mock_setup_dag_for_rapidpe.assert_called_once()
     else:
         with pytest.raises(NotImplementedError):
-            inference.dag_prepare_task(rundir, sid, gid, pipeline, ini).delay()
+            inference.dag_prepare_task(
+                rundir, event, sid, pipeline, frametype_dict).delay()
 
 
 @pytest.mark.parametrize('exc', [condor.JobAborted(1, 'test'),
-                                 condor.JobFailed(1, 'test')])
+                                 condor.JobFailed(1, 'test'),
+                                 condor.JobRunning({'Cluster': 1234})])
 def test_job_error_notification(monkeypatch, tmp_path, exc):
     filenames = ['pe.log', 'pe.err', 'pe.out']
     for filename in filenames:
@@ -304,54 +370,36 @@ def test_job_error_notification(monkeypatch, tmp_path, exc):
             f.write('test')
     upload = Mock()
     monkeypatch.setattr('gwcelery.tasks.gracedb.upload.run', upload)
+    monkeypatch.setattr('subprocess.run', Mock())
 
     inference.job_error_notification(
         None, exc, 'test', 'S1234', str(tmp_path), 'lalinference')
     assert upload.call_count == len(filenames) + 1
 
 
-@pytest.mark.parametrize('pipeline',
-                         ['lalinference', 'bilby', 'my_awesome_pipeline'])
-def test_upload_url(monkeypatch, tmp_path, pipeline):
-    upload = Mock()
-    monkeypatch.setattr('gwcelery.tasks.gracedb.upload.run', upload)
-    if pipeline in ['lalinference', 'bilby']:
-        if pipeline == 'lalinference':
-            path = str(tmp_path / 'posplots.html')
-        else:
-            path = str(tmp_path / 'home.html')
-        with open(path, 'w') as f:
-            f.write('test')
-        inference._upload_url(str(tmp_path), 'G1234', pipeline)
-        upload.assert_called_once()
-    else:
-        with pytest.raises(NotImplementedError):
-            inference._upload_url(str(tmp_path), 'G1234', pipeline)
-
-
-@pytest.mark.parametrize('pipeline',
-                         ['lalinference', 'bilby', 'my_awesome_pipeline'])
+@pytest.mark.parametrize(
+    'pipeline', ['lalinference', 'bilby', 'rapidpe', 'my_awesome_pipeline'])
 def test_dag_finished(monkeypatch, tmp_path, pipeline):
-    gid = 'G1234'
+    sid = 'S1234'
     rundir = str(tmp_path / 'rundir')
     resultdir = str(tmp_path / 'rundir/result')
+    sampledir = str(tmp_path / 'rundir/final_result')
     pe_results_path = str(tmp_path / 'public_html/online_pe')
     monkeypatch.setitem(app.conf, 'pe_results_path', pe_results_path)
-    pe_results_path = os.path.join(pe_results_path, gid, pipeline)
+    pe_results_path = os.path.join(pe_results_path, sid, pipeline)
     os.makedirs(rundir)
     os.makedirs(resultdir)
+    os.makedirs(sampledir)
     os.makedirs(pe_results_path)
 
     upload = Mock()
-    _upload_url = Mock()
     create_label = Mock()
     monkeypatch.setattr('gwcelery.tasks.gracedb.upload.run', upload)
-    monkeypatch.setattr('gwcelery.tasks.inference._upload_url.run',
-                        _upload_url)
     monkeypatch.setattr('gwcelery.tasks.gracedb.create_label.run',
                         create_label)
 
-    if pipeline in ['lalinference', 'bilby']:
+    if pipeline in ['lalinference', 'bilby', 'rapidpe']:
+        kwargs = {}
         if pipeline == 'lalinference':
             paths = [os.path.join(rundir,
                                   'lalinference_1187008756-1187008882.dag'),
@@ -360,32 +408,47 @@ def test_dag_finished(monkeypatch, tmp_path, pipeline):
                      os.path.join(rundir, 'glitch_median_PSD_forLI_V1.dat'),
                      os.path.join(rundir, 'posterior_samples.hdf5'),
                      os.path.join(pe_results_path, 'extrinsic.png'),
-                     os.path.join(pe_results_path, 'sourceFrame.png')]
-        else:
-            paths = [os.path.join(resultdir, 'bilby_merge_result.json'),
+                     os.path.join(pe_results_path, 'intrinsic.png'),
+                     os.path.join(pe_results_path, 'posplots.html')]
+        elif pipeline == 'bilby':
+            input_sample = os.path.join(sampledir, "test_result.hdf5")
+            with open(input_sample, 'wb') as f:
+                f.write(b'result')
+
+            def _subprocess_run(cmd):
+                assert os.path.exists(cmd[1])
+
+            monkeypatch.setattr(
+                'subprocess.run', Mock(side_effect=_subprocess_run))
+
+            kwargs['bilby_mode'] = 'production'
+            paths = [os.path.join(sampledir,
+                                  'Bilby.production.posterior_samples.hdf5'),
                      os.path.join(resultdir, 'bilby_extrinsic_corner.png'),
                      os.path.join(resultdir, 'bilby_intrinsic_corner.png')]
+        else:
+            paths = []
         for path in paths:
             with open(path, 'wb') as f:
                 f.write(b'result')
 
-        inference.dag_finished(rundir, gid, 'S1234', pipeline)
-        assert upload.call_count == len(paths)
-        _upload_url.assert_called_once()
-        if pipeline == 'lalinference':
+        inference.dag_finished(
+            rundir, sid, pipeline, bilby_mode='production')
+
+        if pipeline == 'rapidpe':
+            upload.assert_called_once()
+        else:
+            assert upload.call_count == len(paths)
+
+        if pipeline == 'bilby':
             create_label.assert_called_once()
         else:
             create_label.assert_not_called()
+
         assert not os.path.exists(rundir)
     else:
         with pytest.raises(NotImplementedError):
-            inference.dag_finished(rundir, gid, 'S1234', pipeline)
-
-
-def test_download_psd_failure(monkeypatch):
-    monkeypatch.setattr('gwcelery.tasks.gracedb.download',
-                        Mock(side_effect=HTTPError))
-    assert inference._download_psd('G1234') is None
+            inference.dag_finished(rundir, sid, pipeline)
 
 
 def test_start_pe(monkeypatch, tmp_path):
@@ -411,7 +474,51 @@ def test_start_pe(monkeypatch, tmp_path):
     monkeypatch.setattr('gwcelery.tasks.inference.dag_finished.run',
                         dag_finished)
 
-    inference.start_pe('ini', 'G1234', 'S1234', 'lalinference')
+    inference.start_pe({}, {'graceid': 'G1234'}, 'S1234', 'lalinference')
     dag_prepare_task.assert_called_once()
     condor_submit.assert_called_once()
     dag_finished.assert_called_once()
+
+
+def test_start_pe_multiple_bilby(monkeypatch, tmp_path):
+    path_to_sub = 'pe.dag.condor.sub'
+
+    def mock_dag_prepare_task(
+        rundir, event, superevent_id, pe_pipeline, frametype_dict=None,
+        **kwargs
+    ):
+        assert 'bilby_mode' in kwargs
+
+        @app.task
+        def mock_task():
+            return path_to_sub
+
+        return mock_task.s()
+
+    def mock_condor_submit(path):
+        assert path == path_to_sub
+
+    def mock_dag_finished(rundir, superevent_id, pe_pipeline, **kwargs):
+        assert 'bilby_mode' in kwargs
+
+    dag_prepare_task = Mock(side_effect=mock_dag_prepare_task)
+    condor_submit = Mock(side_effect=mock_condor_submit)
+    dag_finished = Mock(side_effect=mock_dag_finished)
+
+    monkeypatch.setattr('gwcelery.tasks.gracedb.upload.run', Mock())
+    monkeypatch.setattr('distutils.dir_util.mkpath',
+                        Mock(return_value=str(tmp_path)))
+    monkeypatch.setattr('gwcelery.tasks.inference.dag_prepare_task',
+                        dag_prepare_task)
+    monkeypatch.setattr('gwcelery.tasks.condor.submit.run', condor_submit)
+    monkeypatch.setattr('gwcelery.tasks.inference.dag_finished.run',
+                        dag_finished)
+
+    event = {
+        'graceid': 'G1234',
+        'extra_attributes': {'CoincInspiral': {'mchirp': 1}}
+    }
+    inference.start_pe({}, event, 'S1234', 'bilby')
+    assert dag_prepare_task.call_count == 2
+    assert condor_submit.call_count == 2
+    assert dag_finished.call_count == 2
