@@ -1,11 +1,12 @@
-"""A `Nagios plugin <https://nagios-plugins.org/doc/guidelines.html>`_
-for monitoring GWCelery.
+"""A Nagios plugin for monitoring GWCelery.
+
+See https://nagios-plugins.org/doc/guidelines.html.
 """
 from enum import IntEnum
 from sys import exit
 from traceback import format_exc, format_exception
 
-from celery.bin.base import Command
+import click
 import kombu.exceptions
 
 # Make sure that all tasks are registered
@@ -31,9 +32,9 @@ def get_active_queues(inspector):
             for queue in queues}
 
 
-def get_active_lvalert_nodes(inspector):
+def get_active_igwn_alert_topics(inspector):
     return {node for stat in inspector.stats().values()
-            for node in stat.get('lvalert-nodes', ())}
+            for node in stat.get('igwn-alert-topics', ())}
 
 
 def get_expected_queues(app):
@@ -46,8 +47,8 @@ def get_expected_queues(app):
     return result
 
 
-def get_expected_lvalert_nodes(app):
-    return app.conf['lvalert_nodes']
+def get_expected_igwn_alert_topics(app):
+    return app.conf['igwn_alert_topics']
 
 
 def get_active_voevent_peers(inspector):
@@ -58,6 +59,34 @@ def get_active_voevent_peers(inspector):
     return broker_peers, receiver_peers
 
 
+def get_expected_kafka_bootstep_urls(inspector):
+    stats = inspector.stats()
+    expected_kafka_urls = \
+        {peer for stat in stats.values() for peer in
+         stat.get('kafka_topic_up', {})}
+    return expected_kafka_urls
+
+
+def get_active_kafka_bootstep_urls(inspector):
+    stats = inspector.stats()
+    active_kafka_urls = \
+        {kafka_url for stat in stats.values() for kafka_url, active_flag in
+         stat.get('kafka_topic_up', {}).items() if active_flag}
+    return active_kafka_urls
+
+
+def get_undelivered_message_urls(inspector):
+    stats = inspector.stats()
+    undelievered_messages = \
+        {kafka_url for stat in stats.values() for kafka_url, active_flag in
+         stat.get('kafka_delivery_failures', {}).items() if active_flag}
+    return undelievered_messages
+
+
+def get_celery_queue_length(app):
+    return app.backend.client.llen("celery")
+
+
 def check_status(app):
     connection = app.connection()
     try:
@@ -74,16 +103,17 @@ def check_status(app):
         raise NagiosCriticalError('Not all expected queues are active') from \
               AssertionError('Missing queues: ' + ', '.join(missing))
 
-    active = get_active_lvalert_nodes(inspector)
-    expected = get_expected_lvalert_nodes(app)
+    active = get_active_igwn_alert_topics(inspector)
+    expected = get_expected_igwn_alert_topics(app)
     missing = expected - active
     extra = active - expected
     if missing:
-        raise NagiosCriticalError('Not all lvalert nodes are subscribed') \
-            from AssertionError('Missing nodes: ' + ', '.join(missing))
+        raise NagiosCriticalError('Not all IGWN alert topics are subscribed') \
+            from AssertionError('Missing topics: ' + ', '.join(missing))
     if extra:
-        raise NagiosCriticalError('Too many lvalert nodes are subscribed') \
-            from AssertionError('Extra nodes: ' + ', '.join(extra))
+        raise NagiosCriticalError(
+            'Too many IGWN alert topics are subscribed') from AssertionError(
+                    'Extra topics: ' + ', '.join(extra))
 
     broker_peers, receiver_peers = get_active_voevent_peers(inspector)
     if app.conf['voevent_broadcaster_whitelist'] and not broker_peers:
@@ -97,29 +127,47 @@ def check_status(app):
                 from AssertionError('voevent_receiver_address: {}'.format(
                     app.conf['voevent_receiver_address']))
 
+    active = get_active_kafka_bootstep_urls(inspector)
+    expected = get_expected_kafka_bootstep_urls(inspector)
+    missing = expected - active
+    if missing:
+        raise NagiosCriticalError('Not all Kafka bootstep URLs are active') \
+            from AssertionError('Missing urls: ' + ', '.join(missing))
 
-class NagiosCommand(Command):
+    undelivered_messages = get_undelivered_message_urls(inspector)
+    if undelivered_messages:
+        raise NagiosCriticalError(
+            'Not all Kafka messages have been succesfully delivered'
+        ) from AssertionError(
+                'URLs with undelivered messages: ' + ', '.join(missing)
+        )
 
-    def run(self, **kwargs):
-        try:
-            check_status(self.app)
-        except NagiosCriticalError as e:
-            status = NagiosPluginStatus.CRITICAL
-            output, = e.args
-            e = e.__cause__
-            detail = ''.join(format_exception(type(e), e, e.__traceback__))
-        except:  # noqa: E722
-            status = NagiosPluginStatus.UNKNOWN
-            output = 'Unexpected error'
-            detail = format_exc()
-        else:
-            status = NagiosPluginStatus.OK
-            output = 'Running normally'
-            detail = None
-        print('{}: {}'.format(status.name, output))
-        if detail:
-            print(detail)
-        exit(status)
+    celery_queue_length = get_celery_queue_length(app)
+    if celery_queue_length > 50:
+        raise NagiosCriticalError(
+            'Tasks are piled up in Celery queue') from AssertionError(
+                'Length of celery queue is {}'.format(celery_queue_length))
 
 
-NagiosCommand.__doc__ = __doc__
+@click.command(help=__doc__)
+@click.pass_context
+def nagios(ctx):
+    try:
+        check_status(ctx.obj.app)
+    except NagiosCriticalError as e:
+        status = NagiosPluginStatus.CRITICAL
+        output, = e.args
+        e = e.__cause__
+        detail = ''.join(format_exception(type(e), e, e.__traceback__))
+    except:  # noqa: E722
+        status = NagiosPluginStatus.UNKNOWN
+        output = 'Unexpected error'
+        detail = format_exc()
+    else:
+        status = NagiosPluginStatus.OK
+        output = 'Running normally'
+        detail = None
+    print('{}: {}'.format(status.name, output))
+    if detail:
+        print(detail)
+    exit(status)
