@@ -12,6 +12,7 @@ except ImportError:
     import importlib_metadata as metadata
 
 from astropy.time import Time
+from celery import group
 from flask import flash, jsonify, redirect, render_template, request, url_for
 from flask import make_response
 from requests.exceptions import HTTPError
@@ -19,7 +20,8 @@ from requests.exceptions import HTTPError
 from . import app as celery_app
 from ._version import get_versions
 from .flask import app, cache
-from .tasks import first2years, gracedb, orchestrator, circulars, superevents
+from .tasks import first2years, gracedb, orchestrator, circulars, \
+    superevents, first2years_external, external_triggers
 from .util import PromiseProxy
 
 distributions = PromiseProxy(lambda: tuple(metadata.distributions()))
@@ -35,7 +37,8 @@ def index():
         distributions=distributions,
         platform=platform.platform(),
         versions=get_versions(),
-        python_version=sys.version)
+        python_version=sys.version,
+        joint_mdc_freq=celery_app.conf['joint_mdc_freq'])
 
 
 def take_n(n, iterable):
@@ -160,7 +163,7 @@ def typeahead_skymap_filename():
     return jsonify(_search_by_tag_and_filename(
         request.args.get('superevent_id') or '',
         request.args.get('filename') or '',
-        '.fits.gz', 'sky_loc'
+        '.multiorder.fits', 'sky_loc'
     ))
 
 
@@ -186,6 +189,19 @@ def typeahead_p_astro_filename():
     ))
 
 
+@celery_app.task(shared=False, ignore_result=True)
+def _construct_igwn_alert_and_send_prelim_alert(superevent_event_list,
+                                                superevent_id,
+                                                initiate_voevent=True):
+    superevent, event = superevent_event_list
+    alert = {
+        'uid': superevent_id,
+        'object': superevent
+    }
+
+    orchestrator.earlywarning_preliminary_alert(event, alert, superevent_id)
+
+
 @app.route('/send_preliminary_gcn', methods=['POST'])
 def send_preliminary_gcn():
     """Handle submission of preliminary alert form."""
@@ -202,9 +218,13 @@ def send_preliminary_gcn():
             gracedb.update_superevent.si(
                 superevent_id, preferred_event=event_id)
             |
-            gracedb.get_event.si(event_id)
+            group(
+                gracedb.get_superevent.si(superevent_id),
+
+                gracedb.get_event.si(event_id)
+            )
             |
-            orchestrator.preliminary_alert.s(superevent_id)
+            _construct_igwn_alert_and_send_prelim_alert.s(superevent_id)
         ).delay()
         flash('Queued preliminary alert for {}.'.format(superevent_id),
               'success')
@@ -229,10 +249,16 @@ def change_prefered_event():
             gracedb.update_superevent.si(
                 superevent_id, preferred_event=event_id)
             |
-            gracedb.get_event.si(event_id)
+            group(
+                gracedb.get_superevent.si(superevent_id),
+
+                gracedb.get_event.si(event_id)
+            )
             |
-            orchestrator.preliminary_alert.s(
-                superevent_id, initiate_voevent=False)
+            _construct_igwn_alert_and_send_prelim_alert.s(
+                superevent_id,
+                initiate_voevent=False
+            )
         ).delay()
         flash('Changed prefered event for {}.'.format(superevent_id),
               'success')
@@ -282,9 +308,54 @@ def create_update_gcn_circular():
     return redirect(url_for('index'))
 
 
+@app.route('/apply_raven_labels', methods=['POST'])
+def apply_raven_labels():
+    """Applying RAVEN alert label for the missed coincident alerts."""
+    keys = ('superevent_id', 'ext_id', 'event_id')
+    superevent_id, ext_id, event_id, *_ = tuple(request.form.get(key)
+                                                for key in keys)
+    if superevent_id and ext_id and event_id:
+        (
+                gracedb.create_label.si('RAVEN_ALERT', superevent_id)
+                |
+                gracedb.create_label.si('RAVEN_ALERT', ext_id)
+                |
+                gracedb.create_label.si('RAVEN_ALERT', event_id)
+        ).delay()
+        flash('Applied RAVEN alert label for {}.'.format(superevent_id),
+              'success')
+    else:
+        flash('No alert sent. Please fill in all fields.', 'danger')
+    return redirect(url_for('index'))
+
+
 @app.route('/send_mock_event', methods=['POST'])
 def send_mock_event():
     """Handle submission of mock alert form."""
     first2years.upload_event.delay()
     flash('Queued a mock event.', 'success')
     return redirect(url_for('index'))
+
+
+@gracedb.task(shared=False)
+def _create_upload_external_event(gpstime):
+    new_time = first2years_external._offset_time(gpstime)
+
+    ext_event = first2years_external.create_grb_event(new_time, 'Fermi')
+
+    # Upload as from GCN
+    external_triggers.handle_grb_gcn(ext_event)
+
+    return ext_event
+
+
+@app.route('/send_mock_joint_event', methods=['POST'])
+def send_mock_joint_event():
+    """Handle submission of mock alert form."""
+    (
+        first2years.upload_event.si()
+        |
+        _create_upload_external_event.s().set(countdown=5)
+    ).delay()
+    flash('Queued a mock joint event.', 'success')
+    return redirect(url_for('index'))
